@@ -93,7 +93,7 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
     let config = Config::from_args();
     let sturdyref = sturdy::SturdyRef::from_hex(&config.dataspace)?;
     let (i, o) = TcpStream::connect("127.0.0.1:8001").await?.into_split();
-    Actor::new().boot(syndicate::name!("pingpong"), |t| {
+    Actor::new(None).boot(syndicate::name!("pingpong"), |t| {
         relay::connect_stream(t, i, o, false, sturdyref, (), move |_state, t, ds| {
 
             let (send_label, recv_label, report_latency_every, should_echo, bytes_padding) =
