@@ -1,3 +1,5 @@
+use preserves_schema::Codec;
+
 use std::io;
 use std::borrow::Cow;
 use std::path::PathBuf;
@@ -7,6 +9,7 @@ use syndicate::actor::*;
 use syndicate::dataspace::Dataspace;
 use syndicate::during;
 use syndicate::enclose;
+use syndicate::pattern::{lift_literal, drop_literal};
 use syndicate::schemas::dataspace;
 use syndicate::schemas::dataspace_patterns as P;
 use syndicate::schemas::sturdy;
@@ -15,7 +18,6 @@ use syndicate::value::NestedValue;
 use syndicate::value::Record;
 use syndicate::value::Set;
 use syndicate::value::Value;
-use syndicate::value::signed_integer::SignedInteger;
 
 use crate::language::language;
 
@@ -83,6 +85,7 @@ pub enum Expr {
     },
     Dataspace,
     Timestamp,
+    Facet,
 }
 
 #[derive(Debug, Clone)]
@@ -105,12 +108,31 @@ enum Symbolic {
     Bare(String),
 }
 
+struct FacetHandle;
+
 impl<T> Default for Parsed<T> {
     fn default() -> Self {
         Parsed::Skip
     }
 }
 
+impl FacetHandle {
+    fn new() -> Self {
+        FacetHandle
+    }
+}
+
+impl Entity<AnyValue> for FacetHandle {
+    fn message(&mut self, t: &mut Activation, body: AnyValue) -> ActorResult {
+        if let Some("stop") = body.value().as_symbol().map(|s| s.as_str()) {
+            t.stop();
+            return Ok(())
+        }
+        tracing::warn!(?body, "Unrecognised message sent to FacetHandle");
+        return Ok(())
+    }
+}
+
 fn analyze(s: &str) -> Symbolic {
     if s == "_" {
         Symbolic::Discard
@@ -134,7 +156,7 @@ fn discard() -> P::Pattern {
 }
 
 fn dlit(value: AnyValue) -> P::Pattern {
-    P::Pattern::DLit(Box::new(P::DLit { value }))
+    lift_literal(&value)
 }
 
 fn tlit(value: AnyValue) -> sturdy::Template {
@@ -219,40 +241,28 @@ impl<'env> PatternInstantiator<'env> {
             Value::Record(r) => match parse_attenuation(r)? {
                 Some((base_name, alternatives)) =>
                     dlit(self.env.eval_attenuation(base_name, alternatives)?),
-                None => {
-                    // TODO: properly consolidate constant patterns into literals.
-                    match self.instantiate_pattern(r.label())? {
-                        P::Pattern::DLit(b) =>
-                            P::Pattern::DCompound(Box::new(P::DCompound::Rec {
-                                ctor: Box::new(P::CRec {
-                                    label: b.value,
-                                    arity: r.fields().len().into(),
-                                }),
-                                members: r.fields().iter().enumerate()
-                                    .map(|(i, p)| Ok((i.into(), self.instantiate_pattern(p)?)))
-                                    .filter(|e| discard() != e.as_ref().unwrap().1)
-                                    .collect::<io::Result<Map<SignedInteger, P::Pattern>>>()?,
-                            })),
-                        _ => Err(bad_instruction("Record pattern must have literal label"))?,
-                    }
-                },
+                None =>  {
+                    let label = self.instantiate_pattern(r.label())?;
+                    let fields = r.fields().iter().map(|p| self.instantiate_pattern(p))
+                        .collect::<io::Result<Vec<P::Pattern>>>()?;
+                    P::Pattern::DCompound(Box::new(P::DCompound::Rec {
+                        label: drop_literal(&label)
+                            .ok_or(bad_instruction("Record pattern must have literal label"))?,
+                        fields,
+                    }))
+                }
             },
             Value::Sequence(v) =>
                 P::Pattern::DCompound(Box::new(P::DCompound::Arr {
-                    ctor: Box::new(P::CArr {
-                        arity: v.len().into(),
-                    }),
-                    members: v.iter().enumerate()
-                        .map(|(i, p)| Ok((i.into(), self.instantiate_pattern(p)?)))
-                        .filter(|e| discard() != e.as_ref().unwrap().1)
-                        .collect::<io::Result<Map<SignedInteger, P::Pattern>>>()?,
+                    items: v.iter()
+                        .map(|p| self.instantiate_pattern(p))
+                        .collect::<io::Result<Vec<P::Pattern>>>()?,
                 })),
             Value::Set(_) =>
                 Err(bad_instruction(&format!("Sets not permitted in patterns: {:?}", template)))?,
             Value::Dictionary(v) =>
                 P::Pattern::DCompound(Box::new(P::DCompound::Dict {
-                    ctor: Box::new(P::CDict),
-                    members: v.iter()
+                    entries: v.iter()
                         .map(|(a, b)| Ok((a.clone(), self.instantiate_pattern(b)?)))
                         .collect::<io::Result<Map<AnyValue, P::Pattern>>>()?,
                 })),
@@ -421,7 +431,9 @@ impl Env {
                 let (binding_names, pattern) = self.instantiate_pattern(pattern_template)?;
                 let observer = during::entity(self.clone())
                     .on_message(enclose!((binding_names, body) move |env, t, cs: AnyValue| {
-                        env.bind_and_run(t, &binding_names, cs, &*body) }))
+                        t.facet(|t| env.bind_and_run(t, &binding_names, cs, &*body))?;
+                        Ok(())
+                    }))
                     .create_cap(t);
                 self.lookup_target(target)?.assert(t, language(), &dataspace::Observe {
                     pattern,
@@ -459,6 +471,7 @@ impl Env {
             Expr::Template { template } => self.instantiate_value(template),
             Expr::Dataspace => Ok(AnyValue::domain(Cap::new(&t.create(Dataspace::new())))),
             Expr::Timestamp => Ok(AnyValue::new(chrono::Utc::now().to_rfc3339())),
+            Expr::Facet => Ok(AnyValue::domain(Cap::new(&t.create(FacetHandle::new())))),
         }
     }
 
@@ -546,43 +559,29 @@ impl Env {
                     // TODO: properly consolidate constant templates into literals.
                     match self.instantiate_template(binding_names, r.label())? {
                         sturdy::Template::Lit(b) =>
-                            sturdy::Template::TCompound(Box::new(sturdy::TCompound {
-                                ctor: sturdy::ConstructorSpec::CRec(Box::new(sturdy::CRec {
-                                    label: b.value,
-                                    arity: r.fields().len().into(),
-                                })),
-                                members: sturdy::TCompoundMembers(
-                                    r.fields().iter().enumerate()
-                                        .map(|(i, t)| Ok(
-                                            (AnyValue::new(i),
-                                             self.instantiate_template(binding_names, t)?)))
-                                        .collect::<io::Result<Map<_, sturdy::Template>>>()?),
+                            sturdy::Template::TCompound(Box::new(sturdy::TCompound::Rec {
+                                label: b.value,
+                                fields: r.fields().iter()
+                                    .map(|t| self.instantiate_template(binding_names, t))
+                                    .collect::<io::Result<Vec<sturdy::Template>>>()?,
                             })),
                         _ => Err(bad_instruction("Record template must have literal label"))?,
                     }
                 }
             },
             Value::Sequence(v) =>
-                sturdy::Template::TCompound(Box::new(sturdy::TCompound {
-                    ctor: sturdy::ConstructorSpec::CArr(Box::new(sturdy::CArr {
-                        arity: v.len().into(),
-                    })),
-                    members: sturdy::TCompoundMembers(
-                        v.iter().enumerate()
-                            .map(|(i, p)| Ok(
-                                (AnyValue::new(i),
-                                 self.instantiate_template(binding_names, p)?)))
-                            .collect::<io::Result<Map<_, sturdy::Template>>>()?),
+                sturdy::Template::TCompound(Box::new(sturdy::TCompound::Arr {
+                    items: v.iter()
+                        .map(|p| self.instantiate_template(binding_names, p))
+                        .collect::<io::Result<Vec<sturdy::Template>>>()?,
                 })),
             Value::Set(_) =>
                 Err(bad_instruction(&format!("Sets not permitted in templates: {:?}", template)))?,
             Value::Dictionary(v) =>
-                sturdy::Template::TCompound(Box::new(sturdy::TCompound {
-                    ctor: sturdy::ConstructorSpec::CDict(Box::new(sturdy::CDict)),
-                    members: sturdy::TCompoundMembers(
-                        v.iter()
-                            .map(|(a, b)| Ok((a.clone(), self.instantiate_template(binding_names, b)?)))
-                            .collect::<io::Result<Map<_, sturdy::Template>>>()?),
+                sturdy::Template::TCompound(Box::new(sturdy::TCompound::Dict {
+                    entries: v.iter()
+                        .map(|(a, b)| Ok((a.clone(), self.instantiate_template(binding_names, b)?)))
+                        .collect::<io::Result<Map<_, sturdy::Template>>>()?,
                 })),
         })
     }
@@ -595,31 +594,21 @@ fn embed_pattern(p: &P::Pattern) -> sturdy::Pattern {
             pattern: embed_pattern(&b.pattern),
         })),
         P::Pattern::DLit(b) => sturdy::Pattern::Lit(Box::new(sturdy::Lit {
-            value: b.value.clone(),
+            value: language().unparse(&b.value),
         })),
         P::Pattern::DCompound(b) => sturdy::Pattern::PCompound(Box::new(match &**b {
-            P::DCompound::Rec { ctor, members } =>
-                sturdy::PCompound {
-                    ctor: sturdy::ConstructorSpec::CRec(Box::new(sturdy::CRec {
-                        label: ctor.label.clone(),
-                        arity: ctor.arity.clone(),
-                    })),
-                    members: sturdy::PCompoundMembers(
-                        members.iter().map(|(k, v)| (AnyValue::new(k), embed_pattern(v))).collect()),
+            P::DCompound::Rec { label, fields } =>
+                sturdy::PCompound::Rec {
+                    label: label.clone(),
+                    fields: fields.iter().map(embed_pattern).collect(),
                 },
-            P::DCompound::Arr { ctor, members } =>
-                sturdy::PCompound {
-                    ctor: sturdy::ConstructorSpec::CArr(Box::new(sturdy::CArr {
-                        arity: ctor.arity.clone(),
-                    })),
-                    members: sturdy::PCompoundMembers(
-                        members.iter().map(|(k, v)| (AnyValue::new(k), embed_pattern(v))).collect()),
+            P::DCompound::Arr { items } =>
+                sturdy::PCompound::Arr {
+                    items: items.iter().map(embed_pattern).collect(),
                 },
-            P::DCompound::Dict { ctor: _, members } =>
-                sturdy::PCompound {
-                    ctor: sturdy::ConstructorSpec::CDict(Box::new(sturdy::CDict)),
-                    members: sturdy::PCompoundMembers(
-                        members.iter().map(|(k, v)| (k.clone(), embed_pattern(v))).collect()),
+            P::DCompound::Dict { entries } =>
+                sturdy::PCompound::Dict {
+                    entries: entries.iter().map(|(k, v)| (k.clone(), embed_pattern(v))).collect(),
                 },
         })),
     }
@@ -809,6 +798,11 @@ impl<'t> Parser<'t> {
             return Some(Expr::Timestamp);
         }
 
+        if self.peek() == &Value::symbol("facet") {
+            self.drop();
+            return Some(Expr::Facet);
+        }
+
         return Some(Expr::Template{ template: self.shift() });
     }
 }
